
function [indices, gains, warning, nbre_mult] = ...
    algos_mp(F, x, K, algo, version)

% arguments algo et version
% 'MP'  : algo de base
% 'MP+' : algo de base + actualisation des gains  la fin
% 'OMP' : Orthogonal Matching Pursuit
% 'OOMP' ou 'MGS' : Optimized Orthogonal Matching Pursuit
% 'RMGS' : Recursive Modified Gram-Schmidt
% 'CMP'  : Complementary Matching Pursuit
% 'OCMP' : Orthogonal Complementary Matching Pursuit
% 'OOCMP' : Optimized Orthogonal Complementary Matching Pursuit
% 'CRMGS' : Complementary RMGS == OOCMP mais version moins couteuse
% 'GP'   : Gradient Pursuit
% 'LAR'  : Least Angle Regression - recupere de Nicolas
% 'LARS'  : Least Angle Regression - stepwise
% 'RMGS+' ou 'OCMP+' :
%        version 'V1' : orthogonalisation complte
%        version 'V2' : orthogonalisation partielle 1
% 'COSAMP' : alg. Needell and Tropp
% 'SP' : Subspace Pursuit
% 'L1' : L1 minimization

[N, L] = size(F);
indices = ones(K,1);
gains = zeros(K,1);
warning = 0;
Eps = 1e-10;
Val_max = 1e+10;
arret_premature = 0;
switch algo
    case {'MP'} %%% algo de base %%%
        alpha = sum(F.^2);
        beta = x'*F;
        nbre_mult = 2*N*L;
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            gains(k) = beta(indices(k))/alpha(indices(k));
            if k < K
                beta = beta - gains(k)*F(:,indices(k))'*F;
                nbre_mult = nbre_mult + N*L;
                alpha(indices(1:k)) = 1;
                beta(indices(1:k)) = 0;
            end
        end
        
    case {'MP+'} %%% algo de base + actualisation des gains  la fin %%%
        alpha = sum(F.^2);
        beta = x'*F;
        nbre_mult = 2*N*L;
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            gains(k) = beta(indices(k))/alpha(indices(k));
            if k < K
                beta = beta - gains(k)*F(:,indices(k))'*F;
                nbre_mult = nbre_mult + N*L;
                alpha(indices(1:k)) = 1;
                beta(indices(1:k)) = 0;
            end
        end
        A = F(:,indices);
        gains = inv(A'*A)*A'*x;
        nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        
    case 'OMP' %%% Orthogonal Matching Pursuit %%%
        alpha = sum(F.^2);
        beta = x'*F;
        nbre_mult = 2*N*L;
        A = zeros(N,K);
        B = zeros(K,K);
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            A(:,k) = F(:,indices(k));
            if k == 1
                B(1,1) = A(:,1)'*A(:,1);
            else
                B(1:k,k) = A(:,1:k)'*A(:,k);
                B(k,1:k-1) = B(1:k-1,k);
            end
            nbre_mult = nbre_mult + N*k;
            gains(1:k) = inv(B(1:k,1:k))*A(:,1:k)'*x;
            nbre_mult = nbre_mult + k*(k*k/3 + N + k);
            if k < K
                e = x - F(:,indices(1:k))*gains(1:k);
                beta = e'*F;
                nbre_mult = nbre_mult + N*(L+k);
                alpha(indices(1:k)) = 1;
                beta(indices(1:k)) = 0;
            end
        end
        
    case {'OOMP','MGS'} %%% Optimized Orthogonal Matching Pursuit (ou mgs) %%%
        alpha = sum(F.^2);
        beta = x'*F;
        nbre_mult = 2*N*L;
        F_orth = F;
        for k = 1:K
            [temp indices(k)] = max((beta.^2)./alpha);
            if k < K
                q = F_orth(:,indices(k))/sqrt(alpha(indices(k)));
                F_orth = F_orth - q*q'*F_orth;
                nbre_mult = nbre_mult + 2*N*L;
                alpha = sum(F_orth.^2);
                beta = x'*F_orth;
                nbre_mult = nbre_mult + 2*N*L;
                alpha(indices(1:k)) = 1;
                beta(indices(1:k)) = 0;
            end
        end
        A = F(:,indices);
        gains = inv(A'*A)*A'*x;
        nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        
    case 'RMGS'
        alpha = sum(F.^2);
        beta = x'*F;
        nbre_mult = 2*N*L;
        sqrt_alpha = zeros(K,1);
        gains_tilde = zeros(K,1);
        r = zeros(K,L);
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            sqrt_alpha(k) = sqrt(alpha(indices(k)));
            gains_tilde(k) = beta(indices(k))/sqrt_alpha(k);
            if k < K
                temp = r(1:k-1,indices(k))'*r(1:k-1,:);
                r(k,:) = F(:,indices(k))'*F - temp;
                r(k,:) = r(k,:)/sqrt_alpha(k);
                nbre_mult = nbre_mult + L*(N+k-1);
                alpha = alpha - r(k,:).^2;
                beta = beta - gains_tilde(k)*r(k,:);
                nbre_mult = nbre_mult + 2*L;
                alpha(indices(1:k)) = ones(1,k);
                beta(indices(1:k)) = zeros(1,k);
            end
        end
        gains(K) = gains_tilde(K)/sqrt_alpha(K);
        for k = K-1:-1:1
            temp = r(k,indices(k+1:K))*gains(k+1:K);
            gains(k) = (gains_tilde(k) - temp)/sqrt_alpha(k);
        end
        
    case 'CMP' %%% Complementary Matching Pursuit %%%
        B = inv(F*F');
        C = B*F;
        nbre_mult = N*N*N/3 + 2*N*N*L;
        alpha = sum(C.*F);
        beta = x'*C;
        nbre_mult = nbre_mult + 2*N*L;
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            if k < K
                gain = beta(indices(k))/alpha(indices(k));
                beta = beta - gain*F(:,indices(k))'*C;
                nbre_mult = nbre_mult + N*L;
                alpha(indices(1:k)) = 1;
                beta(indices(1:k)) = 0;
            end
        end
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
    case 'OCMP' %%%Orth Complementary Matching Pursuit %%%
        psinv = F'*inv(F*F');
        D = psinv*F;
        nbre_mult = N*N*N/3 + 2*N*N*L + L*L*N;
        % operations precalculees, on peut mettre zero
        alpha = sum(D.*D);
        nbre_mult = nbre_mult + L*L;
        e = psinv*x;
        nbre_mult = nbre_mult + L*N;
        beta = e'*D;
        nbre_mult = nbre_mult + L*L;
        base = zeros(L,K);
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            if k < K
                q = D(:,indices(k))/sqrt(alpha(indices(k)));
                if k > 1
                    for i = 1:k-1
                        q = q - (q'*base(:,i))*base(:,i);
                        nbre_mult = nbre_mult + 2*L;
                    end
                    q = q/norm(q);
                end
                base(:,k) = q;
                
                % e est orthogonal a la base
                e = e - (e'*q)*q;
                nbre_mult = nbre_mult + 2*L;
                beta = e'*D;
                nbre_mult = nbre_mult + L*L;
                beta(indices(1:k)) = 0;
            end
        end
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
    case 'OOCMP' %%% Optimized Orth Complementary Matching Pursuit %%%
        B = inv(F*F');
        C = B*F;
        nbre_mult = N*N*N/3 + 2*N*N*L;
        alpha = sum(C.*F);
        beta = x'*C;
        nbre_mult = nbre_mult + 2*N*L;
        F_orth = C'*F;
        e = C'*x;
        nbre_mult = nbre_mult + N*L*L + N*L;
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            if k < K
                q = F_orth(:,indices(k))/sqrt(alpha(indices(k)));
                F_orth = F_orth - q*q'*F_orth;
                nbre_mult = nbre_mult + L*L + L*L;
                alpha = sum(F_orth.^2);
                beta = e'*F_orth;
                nbre_mult = nbre_mult + 2*L*L;
                alpha(indices(1:k)) = 1;
                beta(indices(1:k)) = 0;
            end
        end
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
    case 'CRMGS' %%% Complementary RMGS %%%
        B = inv(F*F');
        C = B*F;
        nbre_mult = N*N*N/3 + 2*N*N*L;
        
        alpha = sum(C.*F);
        beta = x'*C;
        nbre_mult = nbre_mult + 2*N*L;
        
        sqrt_alpha = zeros(K,1);
        gains_tilde = zeros(K,1);
        r = zeros(K,L);
        
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            sqrt_alpha(k) = sqrt(alpha(indices(k)));
            gains_tilde(k) = beta(indices(k))/sqrt_alpha(k);
            if k < K
                temp = r(1:k-1,indices(k))'*r(1:k-1,:);
                r(k,:) = F(:,indices(k))'*C - temp;
                r(k,:) = r(k,:)/sqrt_alpha(k);
                nbre_mult = nbre_mult + L*(N+k-1);
                alpha = alpha - r(k,:).^2;
                beta = beta - gains_tilde(k)*r(k,:);
                nbre_mult = nbre_mult + 2*L;
                alpha(indices(1:k)) = ones(1,k);
                beta(indices(1:k)) = zeros(1,k);
            end
        end
        
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
    case 'GP' %%% Gradient Pursuit %%%
        alpha = sum(F.^2);
        beta = x'*F;
        nbre_mult = 2*N*L;
        q = zeros(K,K);
        e = x;
        A = zeros(N,K);
        B = zeros(K,K);
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            A(:,k)= F(:,indices(k));
            e_tilde = A(:,1:k)'*e;
            nbre_mult = nbre_mult + N*k;
            q(1:k,k) = e_tilde;
            if k == 1
                B(1,1) = A(:,1)'*A(:,1);
                nbre_mult = nbre_mult + N*k;
            else
                B(1:k,k) = A(:,1:k)'*A(:,k);
                B(k,1:k-1) = B(1:k-1,k);
                nbre_mult = nbre_mult + N*k;
                for i = 1:k-1
                    temp = B(1:k,1:k)*q(1:k,i);
                    den = q(1:k,i)'*temp;
                    if abs(den) < Eps
                        % fprintf('abs(den) < eps k/K = %d/%d \n', k, K)
                        warning = 1;
                        return
                    end
                    q(1:k,k) = q(1:k,k) - (e_tilde'*temp)/den*q(1:k,i);
                    nbre_mult = nbre_mult + k*k + 2*k;
                end
            end
            den = q(1:k,k)'*B(1:k,1:k)*q(1:k,k);
            nbre_mult = nbre_mult + k*k + k;
            if abs(den) < Eps
                % fprintf('abs(den) < eps k/K = %d/%d \n', k, K)
                warning = 1;
                break
            end
            c = (e_tilde'*q(1:k,k))/den;
            gains(1:k) = gains(1:k) + c*q(1:k,k);
            e = x - A(:,1:k)*gains(1:k);
            nbre_mult = nbre_mult + k + k*N;
            if k < K
                beta = e'*F;
                nbre_mult = nbre_mult + N*L;
            end
        end
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
    case 'LAR' %%% Least Angle Regression %%%
        F_norm = F;
        for m = 1:L
            F_norm(:,m) = F_norm(:,m)/norm(F_norm(:,m));
        end
        beta = F_norm'*x;
        nbre_mult = 2*N*L;
        e = x;
        A = zeros(N,K);
        B = zeros(K,K);
        q = zeros(N,K);
        for k = 1:K
            [temp, indices(k)] = max((beta.^2));
            if k > 1 & indices(k) ~= prochain_indice
                fprintf('Pb : ind = %d %d \n', indices(k), prochain_indice)
            end
            
            A(:,k) = sign(beta(indices(k)))*F_norm(:,indices(k));
            if k == 1
                B(1,1) = A(:,1)'*A(:,1);
            else
                B(1:k,k) = A(:,1:k)'*A(:,k);
                B(k,1:k-1) = B(1:k-1,k);
            end
            nbre_mult = nbre_mult + N*k;
            Bm1 = inv(B(1:k,1:k));
            nbre_mult = nbre_mult + k^3/3;
            c = 1/sqrt((ones(1,k)*Bm1*ones(k,1)));
            w = c*Bm1*ones(k,1);
            q(:,k) = A(:,1:k)*w;
            nbre_mult = nbre_mult + k*k + k + N*k;
            
            if k < K
                r = F_norm'*q(:,k);
                nbre_mult = nbre_mult + N*L;
                beta_max = abs(beta(indices(k)));
                g1 = (beta_max-beta)./(c-r);
                g2 = (beta_max+beta)./(c+r);
                g1(find(g1 < 0)) = Val_max;
                g1(indices(1:k)) = Val_max;
                g2(find(g2 < 0)) = Val_max;
                g2(indices(1:k)) = Val_max;
                [g1_min j1_min] = min(g1);
                [g2_min j2_min] = min(g2);
                if g1_min < g2_min
                    g_min = g1_min;
                    prochain_indice = j1_min;
                else
                    g_min = g2_min;
                    prochain_indice = j2_min;
                end
                beta = beta - g_min*r;
                beta(indices(1:k)) = zeros(k,1);
            else
                g_min = q(:,k)'*e;
                nbre_mult = nbre_mult + N;
            end
            e = e - g_min*q(:,k);
            if norm(e) < Eps
                arret_premature = 1;
                break
            end
        end
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
    case 'LARS' %%% Least Angle Regression - stepwise %%%
        Nbre_iter_lar=200; % une grande valeur, sortie de toute facon par 'break'
        DD = F;
        for m = 1:L
            DD(:,m) = DD(:,m)/norm(DD(:,m));
        end
        e = x;
        indices = ones(K,1);
        step = 10*norm(e)/Nbre_iter_lar;  % parametre qui definit precision et complexite
        k = 0;
        nbre_mult = 0; 
        for iter=1:Nbre_iter_lar
            inter_cor = e'*DD;
            nbre_mult = nbre_mult + N*L; 
            [temp, indi] = max(inter_cor.*inter_cor);
            sigi = inter_cor(indi)/abs(inter_cor(indi));
            if k == 0
                k = 1;
                indices(k)=indi;
            end
            
            if k > 0
                logi = 0;
                for p = 1:k
                    if indi == indices(p)
                        logi = 1;
                    end
                end
                if logi == 0
                    k = k+1;
                    indices(k) = indi;
                end
            end
            e = e - DD(:,indi)*step*sigi;
            nbre_mult = nbre_mult + N;  
            
            if k == K
                break
            end
        end
        
        if k < K  % indices aleatoires k...K
            indi=2;
            for kk=1:100
                logi=0;
                for p = 1:k
                    if indi == indices(p)
                        logi=1;
                    end
                end
                if logi==0
                    k=k+1;
                    indices(k)=indi;
                else
                    indi=indi+1;
                end
                if k == K
                    break
                end
            end
        end
        
        % Optimisation des gains connaissant les indices
        % A = F(:,indices(1:k)');
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
        %      x_hat = F(:,indices(1:k)')*gains;
        
    case {'CyRMGS','CyOCMP','CyOOCMP'}
        alpha_init = sum(F.^2);
        beta_init = x'*F;
        % Premire tape : algo RMGS, OCMP ou OOCMP
        switch algo
            case 'CyRMGS' %%% RMGS avec substitution %%%
                alpha = alpha_init;
                beta = beta_init;
                nbre_mult = 2*N*L; % le calcul de alpha_init et beta_init
                % ne doit pas tre comptabilis par OCMP+
                sqrt_alpha = zeros(K,1);
                gains_tilde = zeros(K,1);
                r = zeros(K,L);
                for k = 1:K
                    [temp, indices(k)] = max((beta.^2)./alpha);
                    sqrt_alpha(k) = sqrt(alpha(indices(k)));
                    gains_tilde(k) = beta(indices(k))/sqrt_alpha(k);
                    if k < K
                        temp = r(1:k-1,indices(k))'*r(1:k-1,:);
                        r(k,:) = F(:,indices(k))'*F - temp;
                        r(k,:) = r(k,:)/sqrt_alpha(k);
                        nbre_mult = nbre_mult + L*(N+k-1);
                        alpha = alpha - r(k,:).^2;
                        beta = beta - gains_tilde(k)*r(k,:);
                        nbre_mult = nbre_mult + 2*L;
                        alpha(indices(1:k)) = ones(1,k);
                        beta(indices(1:k)) = zeros(1,k);
                    end
                end
                gains(K) = gains_tilde(K)/sqrt_alpha(K);
                for k = K-1:-1:1
                    temp = r(k,indices(k+1:K))*gains(k+1:K);
                    gains(k) = (gains_tilde(k) - temp)/sqrt_alpha(k);
                end
            case 'CyOCMP' %%% OCMP avec substitution %%%
                psinv = F'*inv(F*F');
                D = psinv*F;
                nbre_mult = N*N*N/3 + 2*N*N*L + L*L*N;
                % operations precalculees, on peut mettre zero
                alpha = sum(D.*D);
                nbre_mult = nbre_mult + L*L;
                e = psinv*x;
                nbre_mult = nbre_mult + L*N;
                beta = e'*D;
                nbre_mult = nbre_mult + L*L;
                base = zeros(L,K);
                for k = 1:K
                    [temp, indices(k)] = max((beta.^2)./alpha);
                    if k < K
                        q = D(:,indices(k))/sqrt(alpha(indices(k)));
                        if k > 1
                            for i = 1:k-1
                                q = q - (q'*base(:,i))*base(:,i);
                                nbre_mult = nbre_mult + 2*L;
                            end
                            q = q/norm(q);
                        end
                        base(:,k) = q;
                        
                        % e est orthogonal a la base
                        e = e - (e'*q)*q;
                        nbre_mult = nbre_mult + 2*L;
                        beta = e'*D;
                        nbre_mult = nbre_mult + L*L;
                        beta(indices(1:k)) = 0;
                    end
                end
                A = F(:,indices);
                if rcond(A'*A) < Eps
                    warning = 1;
                    return
                end
                gains = inv(A'*A)*A'*x;
                nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
                
            case 'CyOOCMP' %%% OOCMP avec substitution %%%
                B = inv(F*F');
                C = B*F;
                nbre_mult = N*N*N/3 + 2*N*N*L;
                
                alpha = sum(C.*F);
                beta = x'*C;
                nbre_mult = nbre_mult + 2*N*L;
                
                sqrt_alpha = zeros(K,1);
                gains_tilde = zeros(K,1);
                r = zeros(K,L);
                
                for k = 1:K
                    [temp, indices(k)] = max((beta.^2)./alpha);
                    sqrt_alpha(k) = sqrt(alpha(indices(k)));
                    gains_tilde(k) = beta(indices(k))/sqrt_alpha(k);
                    if k < K
                        temp = r(1:k-1,indices(k))'*r(1:k-1,:);
                        r(k,:) = F(:,indices(k))'*C - temp;
                        r(k,:) = r(k,:)/sqrt_alpha(k);
                        nbre_mult = nbre_mult + L*(N+k-1);
                        alpha = alpha - r(k,:).^2;
                        beta = beta - gains_tilde(k)*r(k,:);
                        nbre_mult = nbre_mult + 2*L;
                        alpha(indices(1:k)) = ones(1,k);
                        beta(indices(1:k)) = zeros(1,k);
                    end
                end
                
                A = F(:,indices);
                if rcond(A'*A) < Eps
                    warning = 1;
                else
                    gains = inv(A'*A)*A'*x;
                    nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
                end
        end
        
        % Si l'erreur est quasi nulle, tous les vecteurs slectionns
        % sont-ils ncessaires ?
        if 0
            x_hat = F(:,indices)*gains;
            err = (x - x_hat)'*(x - x_hat);
            if err < Eps
                k = 1;
                while k <= length(indices)
                    indices_new = [indices(1:k-1); indices(k+1:end)];
                    A = F(:,indices_new);
                    x_hat = A*inv(A'*A)*A'*x;
                    err = (x - x_hat)'*(x - x_hat);
                    if err < Eps
                        indices = indices_new;
                    else
                        k = k + 1;
                    end
                end
            end
            if length(indices) < K
                fprintf('algo 8 cas 1 : K = %2d -> %2d\n', K, length(indices))
            end
        end
        
        % Deuxime tape : tentative de remplacement d'un vecteur
        % dj slectionn par un autre du dictionnaire
        Nbre_iter_max = 10;
        A = F(:,indices);
        x_hat = A*gains;
        nbre_mult = nbre_mult + N*K;
        xnormax = norm(x_hat);
        
        for no_iter = 1:Nbre_iter_max
            indices_old = indices;
            switch version
                case 'V1' % orthogonalisation complte
                    if K == 1
                        [temp, j_opt] = max((beta_init.^2)./alpha_init);
                        indices_temp = j_opt;
                        gains_temp = beta_init(j_opt)/alpha_init(j_opt);
                        A = F(:,indices_temp);
                        xnorm = norm(A*gains_temp);
                        nbre_mult = nbre_mult + (K+1)*N;
                        if xnorm > xnormax
                            xnormax = xnorm;
                            indices = indices_temp;
                            gains = gains_temp;
                        end
                    else
                        for k = 1:K
                            ind = [indices(1:k-1); indices(k+1:K)];
                            alpha = alpha_init;
                            beta = beta_init;
                            r = zeros(K,L);
                            for p = 1:K-1
                                sqrt_alpha(p) = sqrt(alpha(ind(p)));
                                gains_tilde(p) = beta(ind(p))/sqrt_alpha(p);
                                temp = r(1:p-1,ind(p))'*r(1:p-1,:);
                                r(p,:) = F(:,ind(p))'*F - temp;
                                r(p,:) = r(p,:)/sqrt_alpha(p);
                                nbre_mult = nbre_mult + L*(N+p-1);
                                alpha = alpha - r(p,:).^2;
                                beta = beta - gains_tilde(p)*r(p,:);
                                nbre_mult = nbre_mult + 2*L;
                                alpha(ind(1:p)) = ones(1,p);
                                beta(ind(1:p)) = zeros(1,p);
                            end
                            [temp, j_opt] = max((beta.^2)./alpha);
                            sqrt_alpha(K) = sqrt(alpha(j_opt));
                            gains_tilde(K) = beta(j_opt)/sqrt_alpha(K);
                            indices_temp = [ind; j_opt];
                            gains_temp = zeros(K,1);
                            gains_temp(K) = gains_tilde(K)/sqrt_alpha(K);
                            for p = K-1:-1:1
                                temp = r(p,indices_temp(p+1:K))*gains_temp(p+1:K);
                                gains_temp(p) = (gains_tilde(p) - temp)/sqrt_alpha(p);
                            end
                            A = F(:,indices_temp);
                            xnorm = norm(A*gains_temp);
                            nbre_mult = nbre_mult + (K+1)*N;
                            if xnorm > xnormax
                                xnormax = xnorm;
                                indices = indices_temp;
                                gains = gains_temp;
                            end
                        end
                    end
                    
                case 'V2' % orthogonalisation partielle 1
                    F_orth = F;
                    for k = 1:K
                        q = F_orth(:,indices(k));
                        q = q/norm(q);
                        nbre_mult = nbre_mult + N;
                        F_orth = F_orth - q*q'*F_orth;
                        nbre_mult = nbre_mult + 2*N*L;
                    end
                    alpha = sum(F_orth.^2);
                    nbre_mult = nbre_mult + N*L;
                    if K == 1
                        beta = x'*F_orth;
                        nbre_mult = nbre_mult + N*L;
                        [temp, j_opt] = max((beta.^2)./alpha);
                        indices_temp = j_opt;
                        gains_temp = beta(j_opt)/alpha(j_opt);
                        xnorm = norm(F(:,indices_temp)*gains_temp);
                        nbre_mult = nbre_mult + (K+1)*N;
                        if xnorm > xnormax
                            xnormax = xnorm;
                            indices = indices_temp;
                            gains = gains_temp;
                        end
                    else
                        for k = 1:K
                            ind = indices([(1:k-1), (k+1:K)]);
                            A = F(:,ind);
                            gains_temp = inv(A'*A)*A'*x;
                            nbre_mult = nbre_mult + ...
                                (K-1)*((K-1)*N + (K-1)^2/3 + N + (K-1));
                            x_hat = A*gains_temp;
                            nbre_mult = nbre_mult + N*(K-1);
                            e = x - x_hat;
                            beta = e'*F_orth;
                            nbre_mult = nbre_mult + N*L;
                            alpha(ind) = 1;
                            beta(ind) = 0;
                            [temp, j_opt] = max((beta.^2)./alpha);
                            indices_temp = [ind; j_opt];
                            A = F(:,indices_temp);
                            if rcond(A'*A) < Eps
                                warning = 1;
                                return
                            else
                                gains_temp = inv(A'*A)*A'*x;
                                nbre_mult = nbre_mult + ...
                                    K*(K*N + K^2/3 + N + K);
                            end
                            xnorm = norm(A*gains_temp);
                            nbre_mult = nbre_mult + (K+1)*N;
                            if xnorm > xnormax
                                xnormax = xnorm;
                                indices = indices_temp;
                                gains = gains_temp;
                            end
                        end
                    end                    
            end
            if sum(sort(indices_old) ~= sort(indices)) == 0
                % fprintf(' %s K = %d : indices_old == indices\n', algo, K)
                break
            end
        end
        
        % Une nouvelle fois, si l'erreur est quasi nulle,
        % tous les vecteurs slectionns sont-ils ncessaires ?
        if 0
            A = F(:,indices);
            x_hat = A*inv(A'*A)*A'*x;
            err = (x - x_hat)'*(x - x_hat);
            if err < Eps
                k = 1;
                while k <= length(indices)
                    indices_new = [indices(1:k-1); indices(k+1:end)];
                    A = F(:,indices_new');
                    x_hat = A*inv(A'*A)*A'*x;
                    err = (x - x_hat)'*(x - x_hat);
                    if err < Eps
                        indices = indices_new;
                    else
                        k = k + 1;
                    end
                end
            end
            if length(indices) < P
                fprintf('algo 8 cas 2 : P = %2d -> %2d\n', P, length(indices))
            end
        end
        
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
    case 'COSAMP' %%% algorithme COSAMP %%%
        DD = F;
        for m = 1:L
            DD(:,m) = DD(:,m)/norm(DD(:,m));
        end
        indices = zeros(K,1);
        e = x;
        [lgains,ii] = cosamp_(DD,x,K,0.001,0.0001);
        k = 0;
        for i = 1:L
            if abs(lgains(i)) > 10^(-10)
                k = k+1;
                indices(k) = i;
            end
        end
        
        if k < K  % indices aleatoires k...K
            indi=2;
            for kk=1:100
                logi=0;
                for p = 1:k
                    if indi == indices(p)
                        logi=1;
                    end
                end
                if logi==0
                    k=k+1;
                    indices(k)=indi;
                else
                    indi=indi+1;
                end
                if k == K
                    break
                end
            end
        end
        
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
        end
        
        nbre_mult = ii*cout_inv(N,3*K);  % tres approximativement
        
    case 'SP' %%% algorithme Subspace Pursuit %%%
        DD=F;
        for m = 1:L
            DD(:,m) = DD(:,m)/norm(DD(:,m));
        end
        indices = zeros(K,1);
        e=x;
        [lgains,ii]=sub_pursuit(DD,x,K,0.001,0.0001);
        k=0;
        for i=1:L
            if abs(lgains(i)) > 10^(-10)
                k=k+1;
                indices(k)=i;
            end
        end
        
        if k < K  % indices aleatoires k...K
            indi=2;
            for kk=1:100
                logi=0;
                for p = 1:k
                    if indi == indices(p)
                        logi=1;
                    end
                end
                if logi==0
                    k=k+1;
                    indices(k)=indi;
                else
                    indi=indi+1;
                end
                if k == K
                    break
                end
            end
        end
        
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;;
        end
               
        nbre_mult=ii*(cout_inv(N,2*K)+cout_inv(N,K));  % tres approximativement
        
    case 'L1' %%% algorithme min L1  %%%
        DD=F;
        for m = 1:L
            DD(:,m) = DD(:,m)/norm(DD(:,m));
        end
        indices = zeros(K,1);
        e = x;
        ii = 0;
        lambda=1.;
        lb=log(5);
        la=log(0.01);
        if K > 2
            lambda=0.5;
            lb=log(2);
            la=log(0.01);
        end
        if K >= 4
            lambda=0.1;
            lb=log(1);
            la=log(0.001);
        end
        if K >= 6
            lambda=0.05;
            lb=log(0.1);
            la=log(0.0005);
        end
        if K >= 8
            lambda=0.01;
            lb=log(0.05);
            la=log(0.0001);
        end
        if K >= 10
            lambda=0.005;
            lb=log(0.02);
            la=log(0.0001);
        end
        if K > 12
            lambda=0.0005;
            lb=log(0.001);
            la=log(0.00001);
        end
        
        epsi=0.00001;
        maxiter=100;
        crr=ones(11,1);
        for ij=1:11
            ld=la+(ij-1)*(lb-la)/10;
            crd=bp(F,K,DD,x,L,maxiter,ld,epsi);
            crr(ij)=crd;
        end
        ii=ii+9;
        [err,iin]=min(crr);
        lc=la+(iin-1)*(lb-la)/10;
        lambda=exp(lc);
        lgains=SolveBP(DD,x,L,maxiter,lambda,epsi);
        ii=ii+1;
        kk=0;
        for i=1:L
            if abs(lgains(i)) > 10^(-6)
                kk=kk+1;
            end
        end
        
        [lgai,indii]=sort(abs(lgains),'descend');
        k=min(kk,K);
        lgainss=zeros(size(lgains));
        for i=1:k
            indices(i)=indii(i);
            lgainss(indii(i))=lgains(indii(i));
        end
        
        if k < K  % indices aleatoires k...K
            indi=2;
            for kk=1:100
                logi=0;
                for p = 1:k
                    if indi == indices(p)
                        logi=1;
                    end
                end
                if logi==0
                    k=k+1;
                    indices(k)=indi;
                else
                    indi=indi+1;
                end
                if k == K
                    break
                end
            end
        end
        
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;;
        end
        
        nbre_mult = ii*(0.8*K+6.6)*10^6; %  ii * temps mesure pour 1 execution de solvebp
end

% Vrification des formules suivantes except
% pour tous les algos si warning = 1
% pour l'algo LAR si arret_premature = 1
% pour les algos avec substitution
if ~warning
    a = sum(1:K);
    b = sum((1:K).^2);
    c = sum((1:K).^3);
    switch algo
        case 'MP',   cout = (K+1)*N*L;
        case 'MP+',  cout = (K+1)*N*L + cout_inv(N,K);
        case 'OMP',  cout = (K+1)*N*L + 3*N*sum(1:K-1) + 2*N*K + b + c/3;
        case 'OOMP', cout = (K+1)*N*L + 3*(K-1)*N*L + cout_inv(N,K);
        case 'RMGS', cout = (K+1)*N*L + (K-1)*L*(1+K/2);
        case 'CMP',  cout = (K+1)*N*L + 2*N*N*L + N^3/3 + cout_inv(N,K);
        case 'OCMP',
            cout = N*N*N/3 + 2*N*N*L + L*L*N;
            cout = cout + L*L + L*N + 2*sum(1:K-2)*L;
            cout = cout + K*(2*L + L*L) - 2*L;
            cout = cout + cout_inv(N,K);
        case 'OOCMP',
            cout = N*N*N/3 + 2*N*N*L;
            if 0
                % ancienne formule
                cout = cout + 3*N*L + N*L*L + 4*(K-1)*L*L;
                cout = cout + cout_inv(N,K);
            else
                % nouvelle formule non vrifie (janvier 2011)
                cout = cout + 4*K*N*L;
                cout = cout + cout_inv(N,K);
                nbre_mult = cout;
            end
            
        case 'CRMGS', cout=(K+1)*N*L + 2*N*N*L + N^3/3 + cout_inv(N,K)+ (K-1)*L*(1+K/2);
            
        case 'GP',   cout = (K+1)*N*L + 3*N*a + 2*b + c + cout_inv(N,K);
        case 'LAR',
            if ~arret_premature
                cout = (K+1)*N*L + (2*N+1)*a + b + c/3 + N + cout_inv(N,K);
            else
                cout = nbre_mult;
            end
        case {'LARS','COSAMP','SP','L1'}
            cout = nbre_mult;
        case {'CyRMGS','CyOCMP','CyOOCMP'}
            if 0
                switch algo
                    case 'RMGS+', cout = (K+1)*N*L + (K-1)*L*(1+K/2);
                    case 'OCMP+', cout = 3*N*L + N^3/3 + 2*N*N*L + ...
                            N*L*L + 4*(K-1)*L*L + cout_inv(N,K);
                end
                
                if K > 1
                    cout = cout + N*K + Q*K* ...
                        (N + (K-1)*(3*N*L + N*N) + K*N + cout_inv(N,K));
                end
                cout = cout + cout_inv(N,K);
            else
                cout = nbre_mult;
            end
    end
    
    diff = cout - nbre_mult;
    if abs(diff) >= 1
        %        if algo == 'OCMP'
        %            fprintf(' %s %s %d \n', algo, version, diff)
        %        else
        fprintf(' %s %d \n', algo, diff)
        %        end
    end
end

function cout = cout_inv(N,k)
cout = k*(k*N + k^2/3 + N + k);


